<?php
/**
 * Created by PhpStorm.
 * User: matheussilva
 * Date: 14/07/14
 * Time: 16:43
 */

namespace Importacao\Logic\IO;


class UploadArquivoMastermaq extends UploadArquivo {

    private $caminhoArquivo;

    const EXTENSOES_ACEITAS          = "zip|ZIP";
    const EXTENSOES_ARQUIVOS         = "dbf|DBF";
    const ARQUIVOS_VALIDOS           = "AFAST\d+|BANCO\d+|CLIENTES|FCOES\d+|FERI1\d+|FERI3\d+|FILHO\d+|FUNCS\d+|EV_TEMP|PAY2_\d+|QD_HORAR|RESC3\d+|SALAR\d+|SINDI|TRANS\d+";
    const COMPLETO                   = 1;
    const PERSONALIZADO              = 2;

    function __construct($nrorg)
    {
        parent::__construct($nrorg);
    }


    public function salvaArquivo($caminhoArquivo)
    {
        try {
            $newFileName = $this->getNewFileName($caminhoArquivo);
            $this->validaTransferencia($caminhoArquivo);
            return $this->transfereArquivo($caminhoArquivo, $newFileName);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function transfereArquivo($caminhoArquivo, $nomeNovoArquivo)
    {
        $newDir = APP_PATH."/files/upload/".$this->nrorg."/Mastermaq/";
        if (!file_exists($newDir)) {
            mkdir($newDir, 0777, true);
        }

        $result = explode("/", $caminhoArquivo);
        $oldDir     = UPLOAD_PATH;

        if (rename($oldDir.end($result), "$newDir/$nomeNovoArquivo")) {
            return "$newDir/$nomeNovoArquivo";
        } else {
            return false;
        }
    }

    protected function validaTransferencia($arquivo)
    {
        if (file_exists($arquivo) === false) {
            throw new \Importacao\Logic\IO\Exception\ArquivoNaoEncontrado();
        }

        if (preg_match("/\.(".self::EXTENSOES_ACEITAS.")$/", $arquivo) === false) {
            throw new \Importacao\Logic\IO\Exception\ArquivoNaoSuportado();
        }

    }

    private function validaArquivos($arquivos)
    {
        $arquivos_validos = array();

        foreach ($arquivos as $arquivo ) {
            $arq = explode("\\", $arquivo);
            $arq = end($arq);
            if (mb_ereg("(" . self::ARQUIVOS_VALIDOS . ")\.(". (self::EXTENSOES_ARQUIVOS) .")", $arq) == true) {
                    $arquivos_validos[] = $arquivo;
            }
        }

        return $arquivos_validos;

    }

    private function buscarListaArquivosPastas($path)
    {
        $folder = array($path);

        for($c=0; $c<count($folder); $c++){
            if((is_dir($folder[$c])) && ($folder[$c] != '.') && ($folder[$c] != '..')){
                $files = scandir($folder[$c]);
                foreach ($files as $file) {
                    if(($file != '.') && ($file != '..'))
                        $folder[] = realpath($folder[$c]."/".$file);
//                        $folder[] = realpath($folder[$c].'\/'.$file);
                }
            }
        }

        return $folder;
    }

    private function buscaListaArquivos($path)
    {
        $content = $this->buscarListaArquivosPastas($path);
        $files = array();

        foreach($content as $file){
            if(is_file($file)){
                $files[] = $file;
            }
        }

        return $files;
    }

    public function descompactarArquivo($path, $tipo = self::COMPLETO)
    {
        if ($tipo == self::COMPLETO) {
            $destino = APP_PATH."/files/upload/".$this->nrorg."/Mastermaq/DBF/COMPLETOS";
        } else {
            $destino = APP_PATH."/files/upload/".$this->nrorg."/Mastermaq/DBF/PERSONALIZADOS";
        }

        if (!file_exists($destino)) {
            mkdir($destino , 0777, true);
        }

        $file = new \ZipArchive();
        $result = $file->open($path);

        if ($result == true) {

            $file->extractTo($destino);
            $file->close();

        } else {
            switch($result) {
                case ZipArchive::ER_EXISTS:
                    $ErrMsg = "File already exists.";
                    break;

                case ZipArchive::ER_INCONS:
                    $ErrMsg = "Zip archive inconsistent.";
                    break;

                case ZipArchive::ER_MEMORY:
                    $ErrMsg = "Malloc failure.";
                    break;

                case ZipArchive::ER_NOENT:
                    $ErrMsg = "No such file.";
                    break;

                case ZipArchive::ER_NOZIP:
                    $ErrMsg = "Not a zip archive.";
                    break;

                case ZipArchive::ER_OPEN:
                    $ErrMsg = "Can't open file.";
                    break;

                case ZipArchive::ER_READ:
                    $ErrMsg = "Read error.";
                    break;

                case ZipArchive::ER_SEEK:
                    $ErrMsg = "Seek error.";
                    break;

                default:
                    $ErrMsg = "Error invalid.";
                    break;
            }

            throw new \Exception("Erro: ". $ErrMsg);

        }
        return $destino;
    }

    public function retornaArquivos($caminho, $tipo = self::COMPLETO)
    {
        $local = $this->descompactarArquivo($caminho, $tipo);
        $arquivos = $this->buscaListaArquivos($local);
        $arquivos = $this->validaArquivos($arquivos);
        $arquivos = $this->separaArquivos($arquivos);

        return $arquivos;
    }

    private function separaArquivos($arquivos)
    {
        $arquivosSeparados = array();
        foreach($arquivos as $arquivo){

            switch(true){
                case(preg_match('/(AFAST)/', $arquivo)):{
                    $arquivosSeparados['AFAST'] = $arquivo;
                    break;
                }
                case(preg_match('/(BANCO)/', $arquivo)):{
                    $arquivosSeparados['BANCO'] = $arquivo;
                    break;
                }
                case(preg_match('/(CLIENTES)/', $arquivo)):{
                    $arquivosSeparados['CLIENTES'] = $arquivo;
                    break;
                }
                case(preg_match('/(FCOES)/', $arquivo)):{
                    $arquivosSeparados['FCOES'] = $arquivo;
                    break;
                }
                case(preg_match('/(FERI1)/', $arquivo)):{
                    $arquivosSeparados['FERI1'] = $arquivo;
                    break;
                }
                case(preg_match('/(FERI3)/', $arquivo)):{
                    $arquivosSeparados['FERI3'] = $arquivo;
                    break;
                }
                case(preg_match('/(FILHO)/', $arquivo)):{
                    $arquivosSeparados['FILHO'] = $arquivo;
                    break;
                }
                case(preg_match('/(FUNCS)/', $arquivo)):{
                    $arquivosSeparados['FUNCS'] = $arquivo;
                    break;
                }
                case(preg_match('/(EV_TEMP)/', $arquivo)):{
                    $arquivosSeparados['EV_TEMP'] = $arquivo;
                    break;
                }
                case(preg_match('/(PAY2_)/', $arquivo)):{
                    $arquivosSeparados['PAY2_'] = $arquivo;
                    break;
                }
                case(preg_match('/(QD_HORAR)/', $arquivo)):{
                    $arquivosSeparados['QD_HORAR'] = $arquivo;
                    break;
                }
                case(preg_match('/(RESC3)/', $arquivo)):{
                    $arquivosSeparados['RESC3'] = $arquivo;
                    break;
                }
                case(preg_match('/(SALAR)/', $arquivo)):{
                    $arquivosSeparados['SALAR'] = $arquivo;
                    break;
                }
                case(preg_match('/(SINDI)/', $arquivo)):{
                    $arquivosSeparados['SINDI'] = $arquivo;
                    break;
                }
                case(preg_match('/(TRANS)/', $arquivo)):{
                    $arquivosSeparados['TRANS'] = $arquivo;
                    break;
                }
            }
        }

        return $arquivosSeparados;
    }

    public function limparDiretrio($nrorg)
    {
        $destino = realpath(APP_PATH."/files/upload/".$nrorg."/Mastermaq/DBF")." /S /Q";
//        system('rmdir /S /Q "C:\\workfolder\\folha\\products\\projects\\importacao\\files\\upload\\13\\Mastermaq\\DBF"');

        $arquivos = $this->buscaListaArquivos($destino);
        foreach($arquivos as $arquivo){
            unlink($arquivo);
        }

        $pastas = $this->buscarListaArquivosPastas($destino);
        foreach($pastas as $pasta){
            if(is_dir($pasta) && ($pasta != '.') && ($pasta != '..')){
                rmdir($pasta);
            }
        }
    }

    public function compactarArquivos($caminho)
    {
        $arquivos = $this->buscaListaArquivos($caminho);
        $arquivoZip = $caminho."/Arquivos_Migracao_HRM.zip";
        $zip = new \ZipArchive();
        $result = $zip->open($arquivoZip, \ZipArchive::CREATE);

        if ($result === true) {
            foreach($arquivos as $arquivo){
                $filename = explode("\\", $arquivo);
                if(!$zip->addFile($arquivo, end($filename))){
                    throw new \Exception("Falha ao adicionar ao arquivo.");
                }
            }
        } else {
            throw new \Exception("Falha ao compactar os arquivos, contate o administrador.");
        }

        $zip->close();

        tk_curl_download($arquivoZip);
    }

} 